
/*
  IRsend, IRsendRaw routines written by Brian Millier Sept 22,2017
  (based upon the NEC send routines written by espressif and found on their github site)
  IRsendRaw takes raw pulse/space times found in EEPROM, loads the RMT RAM, and sends this code out (uses 6 RMT transmitter blocks and 1 RMT receiver block) 
*/

#include <stdio.h>
#include <string.h>

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_system.h"
#include "nvs_flash.h"
#include "driver/periph_ctrl.h"

#include "freertos/queue.h"
#include "nvs_flash.h"

#include "esp32_rmt.h"
#include <EEPROM.h>




//send 1 frame of IR data, repeated the number of times set in min_repeat param

// may have to check out this routine again now that parts of rmt.cpp, h and esp32_rmt.cpp have been modified to work with IRsendRaw

void ESP32_RMT::IRSend(int freq,int phead,int shead, int plead, int pre_data_bits,unsigned int pre_data,
int ppre,int spre, int bits,unsigned int ButtonCode,int ppost, int spost, int post_data_bits,unsigned int post_data, int pzero,
int szero, int pone,int sone,int ptrail, int pfoot,int sfoot,int gap,int min_repeat)
{
	for (int m = 0; m < (min_repeat + 1); m++) {

		int channel = txChannel;     // was  RMT_TX_CHANNEL;
		size_t size = (sizeof(rmt_item32_t) * 128);   // allow for a maximum of 128 items or 128 pulse & space timing params 
		//each item represent a cycle of waveform.
		rmt_item32_t* item = (rmt_item32_t*)malloc(size);
		rmt_item32_t* itemSave = item;   // itemSave is a duplicate pointer, needed later on for transmitting
		int item_num = 0;
		int j;
		
		IR_fill_item_level(item++, phead, shead);
		item_num++;
		// Leading pulse (NO SPACE)
		if (plead > 0) {
		//	int space = 10;
			IR_fill_item_pulse(item++, plead);  
			item_num++;
		}
		unsigned int  mask, v, w;
		// pre-data parameter
		if (pre_data_bits > 0) {
			mask = 2147483648;  // pre data value was stored, left justified in a 32 bit unsigned integer var
			for (j = 0; j < pre_data_bits; j++) {
				v = pre_data & mask;
				if (v == 0) {
					IR_fill_item_bit_zero(item++, pzero, szero);
				}
				else {
					IR_fill_item_bit_one(item++, pone, sone);
				}
				mask >>= 1;
				item_num++;
			}
		}
		// post pre_data pulse/space
		if (ppre > 0) {
			IR_fill_item_level(item++, ppre, spre);
			item_num++;
		}
		// button Code can be up to 32 bits long, and is packed, left justified, into a 32 bit unsigned integer
		
		mask = 2147483648;   // 1 << 31
		for (j = 0; j < bits; j++) {
			v = ButtonCode & mask;
			if (v == 0) {
				IR_fill_item_bit_zero(item++, pzero, szero);
			}
			else {
				IR_fill_item_bit_one(item++, pone, sone);
			}
			mask >>= 1;
			item_num++;
		}
		// post button code pulse/space
		if (ppost > 0) {
			IR_fill_item_level(item++, ppost, spost);
			item_num++;
		}
		// post data
		if (post_data_bits > 0) {
			mask = 2147483648;  // post data value was stored, left justified in a 32 bit unsigned integer var
			for (j = 0; j < post_data_bits; j++) {
				v = post_data & mask;
				if (v == 0) {
					IR_fill_item_bit_zero(item++, pzero, szero);
				}
				else {
					IR_fill_item_bit_one(item++, pone, sone);
				}
				mask >>= 1;
				item_num++;
			}
		}
		// trailing pulse (NO SPACE)
		if (ptrail > 0) {
			//int space = 10;
			IR_fill_item_pulse(item++, ptrail); 
			item_num++;
		}
		// footer pulse
		if (pfoot > 0) {
			IR_fill_item_level(item++, pfoot, sfoot);
			item_num++;
		}
		if (gap > 0) {
			unsigned int g = gap >> 4;   // gap can be a lot longer than 32767 * tickus (0.8 us) break it down into 8 shorter segments
			for (int l = 0; l < 16; l++) {
				IR_fill_item_gap(item++, g);
				item_num++;
			}

		}
		item = itemSave;  // must restore item pointer back to start of RMT memory array
		// send data according to the waveform items.
		rmt_write_items((rmt_channel_t)channel, item, item_num, true);
		//now free the data block
		free(item);
	}
}

  /* this sends the raw pulse/space code found in EEPROM record which starts at address of "addr" passed into routine
  the pulses are stored as uint16_t with MSB bit high
  the spaces are stored as uint16_t with MSB low
  any pulse > 32510 us long is broken down into more than one pulse (leading one(s) are 32510 until the residue is < 32510 )
  same applies to long spaces (gaps)
  6 blocks are used for the tranmsitter (384 items)
  1 block only is used for the receiver   (64 items) as reciever is only used to sample first sync carrier samples to determine an accurate carrier freq.
  */
void ESP32_RMT::IRSendRaw(uint8_t *  addr)  // addr points to the uint8_t buf[] array
{
	int channel = txChannel;     // was  RMT_TX_CHANNEL;
	size_t size = (sizeof(rmt_item32_t) * 384);   // allow for a maximum of  384 items of  pulse & space timing params  ( six 64 word RMT memory blocks)
												  //each item represents 1 pulse/space or trailing pulse, elongated pulse (2 pulses) or elongated space (2 spaces)
	rmt_item32_t* item = (rmt_item32_t*)malloc(size);
	rmt_item32_t* itemSave = item;   // itemSave is a duplicate pointer, needed later on for transmitting
	int item_num = 0;
	int j;
	uint16_t value, pulse1, space1, pulse2, space2;
	uint16_t MSB, LSB, addrSave;
//	addrSave = addr;
	while (1) {
		if (item_num > 383) break;   // even if < 256 pulse/space combos are stored in EEPROM, when there are long gaps/pulses, the expansion of
									 // of these into the RMT items may exceed the 383 items available (highly unlikely)	
		// read in two 16 bit integer values from buf[] array 
		space1 = 0; pulse1 = 0;
		pulse2 = 0; space2 = 0;
		MSB = *addr++;
		LSB = *addr++;
		value = (MSB << 8) + LSB;
		if (value > 32767) {
			// its a pulse
			pulse1 = value;
		}
		else {
			//its a space
			space1 = value;
		}
		MSB = *addr++;
		LSB = *addr++;
		value = (MSB << 8) + LSB;
		if (value > 32767) {
			// its a pulse
			pulse2 = value;
		}
		else {
			// its a space
			space2 = value;
		}
		if (pulse1 == 0 && pulse2 == 0 && space1 == 0 and space2 == 0) {
			break;   // we're at end of record
		}


		if (pulse1 > 0 && space2 > 0) {         // normal pulse/space combination
			IR_fill_item_level(item++, (int)pulse1, (int)space2);
			item_num++;
		}
		if (pulse1 > 0 && pulse2 > 0) {      // long pulse   
			IR_fill_item_doublePulse(item++, (int)pulse1, (int)pulse2);
			item_num++;
		}
		if (space1 > 0 && space2 > 0) {     // long space (gap)   
			IR_fill_item_doubleSpace(item++, (int)space1, (int)space2);
			item_num++;
		}
		if (pulse1 > 0 && space2 == 0) {   // trailing pulse
			IR_fill_item_pulse(item++, (int)pulse1);
			item_num++;
		}
		if (space1 > 0 && pulse2 > 0) {   //elongated space interval, followed by a pulse 
			IR_fill_item_gap(item++, space1);
			IR_fill_item_pulse(item++, pulse2);
			item_num++;
			item_num++;
		}
	}
	//	printf("RMT addr = %d \n", addr - addrSave);
		item = itemSave;  // must restore item pointer back to start of RMT memory array
						  // send data according to the waveform items.
		rmt_write_items((rmt_channel_t)channel, item, item_num, true);
		//now free the data block
		free(item);
	
}




inline void ESP32_RMT::IR_fill_item_level(rmt_item32_t* item, int high_us, int low_us)
{
    item->level0 = 1;
    item->duration0 = (high_us) ;
    item->level1 = 0;
    item->duration1 = (low_us) ;
}

inline void ESP32_RMT::IR_fill_item_gap(rmt_item32_t* item,  int low_us)
{
	item->level0 = 0;
	item->duration0 = 4;    // was 1 *RMT_TICK_10_US;
	item->level1 = 0;
	item->duration1 = (low_us-4);
}

inline void ESP32_RMT::IR_fill_item_pulse(rmt_item32_t* item, int high_us)
{
	item->level0 = 1;
	item->duration0 = 4;    
	item->level1 = 1;
	item->duration1 = (high_us - 4);}

inline void ESP32_RMT::IR_fill_item_doublePulse(rmt_item32_t* item, int high_us, int high_us2)
{
	item->level0 = 1;
	item->duration0 = high_us;   
	item->level1 = 1;
	item->duration1 = high_us2;

}

inline void ESP32_RMT::IR_fill_item_doubleSpace(rmt_item32_t* item, int low_us, int low_us2)
{
	item->level0 = 0;
	item->duration0 = low_us;   
	item->level1 = 0;
	item->duration1 = low_us2;

}

 void ESP32_RMT::IR_fill_item_header(rmt_item32_t* item,int phead,int shead)
{
    IR_fill_item_level(item, phead, shead);
}

 void ESP32_RMT::IR_fill_item_bit_one(rmt_item32_t* item,int pulseUs, int spaceUs)
{
    IR_fill_item_level(item, pulseUs, spaceUs);
}

void ESP32_RMT::IR_fill_item_bit_zero(rmt_item32_t* item, int pulseUs, int spaceUs)
{
    IR_fill_item_level(item, pulseUs, spaceUs);
}

